/*
 * $QNXLicenseC: 
 * Copyright 2011, QNX Software Systems.  
 *  
 * Licensed under the Apache License, Version 2.0 (the "License"). You  
 * may not reproduce, modify or distribute this software except in  
 * compliance with the License. You may obtain a copy of the License  
 * at: http://www.apache.org/licenses/LICENSE-2.0  
 *  
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" basis,  
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied. 
 * 
 * This file may contain contributions from others, either as  
 * contributors under the License or as licensors under other terms.   
 * Please review this entire file for other proprietary rights or license  
 * notices, as well as the QNX Development Suite License Guide at  
 * http://licensing.qnx.com/license-guide/ for other information. 
 * $
 */

/*
 * Routines to initialize the various hardware subsystems
 * on the i.MX6Q Sabre-Lite
 */

#include "startup.h"
#include "board.h"

#define CCM_CCOSR_CLKO1_EN  (0x1 << 7)
#define CCM_CCOSR_CLKO1_DIV_8   (0x7 << 4)
#define CCM_CCOSR_CLKO1_AHB (0xb << 0)

#define DEBUG_ON  0

void mx6sdl_init_i2c1(void)
{

#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif

    /* I2C1  SCL */
    pinmux_set_swmux(SWMUX_SDL_EIM_D21, MUX_CTL_MUX_MODE_ALT6 | MUX_CTL_SION);
    pinmux_set_padcfg(SWPAD_SDL_EIM_D21, MX6X_PAD_SETTINGS_I2C);
    pinmux_set_input(SWINPUT_SDL_I2C1_IPP_SCL_IN, 0x0);

    /* I2C1  SDA */
    pinmux_set_swmux(SWMUX_SDL_EIM_D28, MUX_CTL_MUX_MODE_ALT1 | MUX_CTL_SION);
    pinmux_set_padcfg(SWPAD_SDL_EIM_D28, MX6X_PAD_SETTINGS_I2C);
    pinmux_set_input(SWINPUT_SDL_I2C1_IPP_SDA_IN, 0x0);
}

void mx6sdl_init_i2c2(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    /* I2C2  SCL */
    pinmux_set_swmux(SWMUX_SDL_KEY_COL3, MUX_CTL_MUX_MODE_ALT4 | MUX_CTL_SION);
    pinmux_set_padcfg(SWPAD_SDL_KEY_COL3, MX6X_PAD_SETTINGS_I2C);
    pinmux_set_input(SWINPUT_SDL_I2C2_IPP_SCL_IN, 0x1);

    /* I2C2  SDA */
    pinmux_set_swmux(SWMUX_SDL_KEY_ROW3, MUX_CTL_MUX_MODE_ALT4 | MUX_CTL_SION);
    pinmux_set_padcfg(SWPAD_SDL_KEY_ROW3, MX6X_PAD_SETTINGS_I2C);
    pinmux_set_input(SWINPUT_SDL_I2C2_IPP_SDA_IN, 0x1);
}

void mx6sdl_init_i2c3(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
   // i2c3 Not used on TS boards
   
   /* I2C3 SCL */
   pinmux_set_swmux(SWMUX_SDL_GPIO_5, MUX_CTL_MUX_MODE_ALT6 | MUX_CTL_SION);
   pinmux_set_padcfg(SWPAD_SDL_GPIO_5, MX6X_PAD_SETTINGS_I2C);
   pinmux_set_input(SWINPUT_SDL_I2C3_IPP_SCL_IN, 0x2);

   /* I2C3  SDA */
   pinmux_set_swmux(SWMUX_SDL_GPIO_16, MUX_CTL_MUX_MODE_ALT6 | MUX_CTL_SION);
   pinmux_set_padcfg(SWPAD_SDL_GPIO_16, MX6X_PAD_SETTINGS_I2C);
   pinmux_set_input(SWINPUT_SDL_I2C3_IPP_SDA_IN, 0x2);
}

void mx6sdl_init_audmux_pins(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    // CCM CLKO pin muxing
    pinmux_set_swmux(SWMUX_SDL_GPIO_0, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_GPIO_0, MX6X_PAD_SETTINGS_CLKO);

    // FIX ME different on TS boards 
    // AUDMUX pin muxing (AUD4_RXD, AUD4_TXC, AUD4_TXD, AUD4_TXFS)  
    pinmux_set_swmux(SWMUX_SDL_SD2_DAT0, MUX_CTL_MUX_MODE_ALT3);
    pinmux_set_input(SWINPUT_SDL_AUDMUX_P4_DA_AMX, 0x0);

    pinmux_set_swmux(SWMUX_SDL_SD2_DAT3, MUX_CTL_MUX_MODE_ALT3);
    pinmux_set_input(SWINPUT_SDL_AUDMUX_P4_TXCLK_AMX, 0x1);

    pinmux_set_swmux(SWMUX_SDL_SD2_DAT2, MUX_CTL_MUX_MODE_ALT3);
    pinmux_set_input(SWINPUT_SDL_AUDMUX_P4_DB_AMX, 0x0);

    pinmux_set_swmux(SWMUX_SDL_SD2_DAT1, MUX_CTL_MUX_MODE_ALT3);
    pinmux_set_input(SWINPUT_SDL_AUDMUX_P4_TXFS_AMX, 0x0);

    // Configure CLKO to produce 16.5MHz master clock
    out32(MX6X_CCM_BASE + MX6X_CCM_CCOSR, CCM_CCOSR_CLKO1_EN | CCM_CCOSR_CLKO1_AHB | CCM_CCOSR_CLKO1_DIV_8); 
    
}
void mx6sdl_init_enet(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    // RGMII MDIO - transfers control info between MAC and PHY
    pinmux_set_swmux(SWMUX_SDL_ENET_MDIO, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_input(SWINPUT_SDL_ENET_IPP_IND_MAC0_MDIO,0);

    // RGMII MDC - output from MAC to PHY, provides clock reference for MDIO
    pinmux_set_swmux(SWMUX_SDL_ENET_MDC, MUX_CTL_MUX_MODE_ALT1);

    // RGMII TXC - output from MAC, provides clock used by RGMII_TXD[3:0], RGMII_TX_CTL
    pinmux_set_swmux(SWMUX_SDL_RGMII_TXC, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_TXC, MX6X_PAD_SETTINGS_ENET);

    // RGMII TXD[3:0] - Transmit Data Output
    pinmux_set_swmux(SWMUX_SDL_RGMII_TD0, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_TD0, MX6X_PAD_SETTINGS_ENET);

    pinmux_set_swmux(SWMUX_SDL_RGMII_TD1, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_TD1, MX6X_PAD_SETTINGS_ENET);

    pinmux_set_swmux(SWMUX_SDL_RGMII_TD2, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_TD2, MX6X_PAD_SETTINGS_ENET);

    pinmux_set_swmux(SWMUX_SDL_RGMII_TD3, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_TD3, MX6X_PAD_SETTINGS_ENET);


    pinmux_set_swmux(SWMUX_SDL_RGMII_RXC, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_RXC, MX6X_PAD_SETTINGS_ENET);
    pinmux_set_input(SWINPUT_SDL_ENET_IPP_IND_MAC0_RXCLK,0);

    pinmux_set_swmux(SWMUX_SDL_RGMII_RD0, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_RD0, MX6X_PAD_SETTINGS_ENET);
    pinmux_set_input(SWINPUT_SDL_ENET_IPP_IND_MAC_RXDATA_0,0);

    pinmux_set_swmux(SWMUX_SDL_RGMII_RD1, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_RD1, MX6X_PAD_SETTINGS_ENET);
    pinmux_set_input(SWINPUT_SDL_ENET_IPP_IND_MAC_RXDATA_1,0);

    pinmux_set_swmux(SWMUX_SDL_RGMII_RD2, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_RD2, MX6X_PAD_SETTINGS_ENET);
    pinmux_set_input(SWINPUT_SDL_ENET_IPP_IND_MAC_RXDATA_2,0);

    pinmux_set_swmux(SWMUX_SDL_RGMII_RD3, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_RD3, MX6X_PAD_SETTINGS_ENET);
    pinmux_set_input(SWINPUT_SDL_ENET_IPP_IND_MAC_RXDATA_3,0);

    pinmux_set_swmux(SWMUX_SDL_RGMII_RX_CTL, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_RX_CTL, MX6X_PAD_SETTINGS_ENET);
    pinmux_set_input(SWINPUT_SDL_ENET_IPP_IND_MAC0_RXEN,0);


    // RGMII TX_CTL - contains TXEN on TXC rising edge, TXEN XOR TXERR on TXC falling edge
    pinmux_set_swmux(SWMUX_SDL_RGMII_TX_CTL, MUX_CTL_MUX_MODE_ALT1);
    pinmux_set_padcfg(SWPAD_SDL_RGMII_TX_CTL, MX6X_PAD_SETTINGS_ENET);

    // set ENET_REF_CLK to mux mode 1 - TX_CLK, this is a 125MHz input which is driven by the PHY
    pinmux_set_swmux(SWMUX_SDL_ENET_REF_CLK, MUX_CTL_MUX_MODE_ALT1);    

    // TX EN set to GPIO1[28]
    pinmux_set_swmux(SWMUX_SDL_ENET_TX_EN, MUX_CTL_MUX_MODE_ALT5);
    
    // RGMII reset GPIO4[20]
    pinmux_set_swmux(SWMUX_SDL_DI0_PIN4, MUX_CTL_MUX_MODE_ALT5);
    pinmux_set_padcfg(SWPAD_SDL_DI0_PIN4, MX6X_PAD_SETTINGS_GPIO);


    /* Micrel KSZ9031RN requires 10ms reset time */
    mx6x_reset_gpio_pin_fin(10000);
}


/* Wifi */
void mx6sdl_init_usdhc1_sdio(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    /* SD1 CLK */
    pinmux_set_swmux(SWMUX_SDL_SD1_CLK, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD1_CLK, MX6X_PAD_SETTINGS_USDHC_SDIO);

    /* SD1 CMD */
    pinmux_set_swmux(SWMUX_SDL_SD1_CMD, MUX_CTL_MUX_MODE_ALT0 | MUX_CTL_SION);
    pinmux_set_padcfg(SWPAD_SDL_SD1_CMD, MX6X_PAD_SETTINGS_USDHC_SDIO);

    /* SD1 DAT0 */
    pinmux_set_swmux(SWMUX_SDL_SD1_DAT0, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD1_DAT0, MX6X_PAD_SETTINGS_USDHC_SDIO);

    /* SD1 DAT1 */
    pinmux_set_swmux(SWMUX_SDL_SD1_DAT1, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD1_DAT1, MX6X_PAD_SETTINGS_USDHC_SDIO);

    /* SD1 DAT2 */
    pinmux_set_swmux(SWMUX_SDL_SD1_DAT2, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD1_DAT2, MX6X_PAD_SETTINGS_USDHC_SDIO);

    /* SD1 DAT3 */
    pinmux_set_swmux(SWMUX_SDL_SD1_DAT3, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD1_DAT3, MX6X_PAD_SETTINGS_USDHC_SDIO);

    /* Wifi IRQ, GPIO1_26 */
    pinmux_set_swmux(SWMUX_SDL_ENET_RXD1, MUX_CTL_MUX_MODE_ALT5);
    pinmux_set_padcfg(SWPAD_SDL_ENET_RXD1, MX6X_PAD_SETTINGS_GPIO);
    
}


/* TS-4900 SD card */
void mx6sdl_init_usdhc2(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    /* SD2 CLK */
    pinmux_set_swmux(SWMUX_SDL_SD2_CLK, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD2_CLK, MX6X_PAD_SETTINGS_USDHC);

    /* SD2 CMD */
    pinmux_set_swmux(SWMUX_SDL_SD2_CMD, MUX_CTL_MUX_MODE_ALT0 | MUX_CTL_SION);
    pinmux_set_padcfg(SWPAD_SDL_SD2_CMD, MX6X_PAD_SETTINGS_USDHC);

    /* SD2 DAT0 */
    pinmux_set_swmux(SWMUX_SDL_SD2_DAT0, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD2_DAT0, MX6X_PAD_SETTINGS_USDHC);

    /* SD2 DAT1 */
    pinmux_set_swmux(SWMUX_SDL_SD2_DAT1, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD2_DAT1, MX6X_PAD_SETTINGS_USDHC);

    /* SD2 DAT2 */
    pinmux_set_swmux(SWMUX_SDL_SD2_DAT2, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD2_DAT2, MX6X_PAD_SETTINGS_USDHC);

    /* SD2 DAT3 */
    pinmux_set_swmux(SWMUX_SDL_SD2_DAT3, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD2_DAT3, MX6X_PAD_SETTINGS_USDHC);

    // EN_SD_POWER# on EIM_EB0, ALT5 = GPIO2[28]
    pinmux_set_swmux(SWMUX_SDL_EIM_EB0, MUX_CTL_MUX_MODE_ALT5);
    pinmux_set_padcfg(SWPAD_SDL_EIM_EB0, MX6X_PAD_SETTINGS_GPIO);   
    
    out32(MX6X_GPIO2_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO2_BASE + MX6X_GPIO_GDIR) | (0x1 << 28));
    out32(MX6X_GPIO2_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO2_BASE + MX6X_GPIO_DR) & ~(0x1 << 28));
}

/* On the TS-4900, SD3 is connected to the eMMC chip */ 
void mx6sdl_init_usdhc3(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    /* SD3 CLK */
    pinmux_set_swmux(SWMUX_SDL_SD3_CLK, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD3_CLK, MX6X_PAD_SETTINGS_USDHC);

    /* SD3 CMD */
    pinmux_set_swmux(SWMUX_SDL_SD3_CMD, MUX_CTL_MUX_MODE_ALT0 | MUX_CTL_SION);
    pinmux_set_padcfg(SWPAD_SDL_SD3_CMD, MX6X_PAD_SETTINGS_USDHC);

    /* SD3 DAT0 */
    pinmux_set_swmux(SWMUX_SDL_SD3_DAT0, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD3_DAT0, MX6X_PAD_SETTINGS_USDHC);

    /* SD3 DAT1 */
    pinmux_set_swmux(SWMUX_SDL_SD3_DAT1, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD3_DAT1, MX6X_PAD_SETTINGS_USDHC);

    /* SD3 DAT2 */
    pinmux_set_swmux(SWMUX_SDL_SD3_DAT2, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD3_DAT2, MX6X_PAD_SETTINGS_USDHC);

    /* SD3 DAT3 */
    pinmux_set_swmux(SWMUX_SDL_SD3_DAT3, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_padcfg(SWPAD_SDL_SD3_DAT3, MX6X_PAD_SETTINGS_USDHC);
    
}



void mx6sdl_init_ecspi(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
   /* SPI_1_CLK */
   pinmux_set_swmux(SWMUX_SDL_EIM_D16, MUX_CTL_MUX_MODE_ALT1);
   pinmux_set_padcfg(SWPAD_SDL_EIM_D16, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI1_IPP_CSPI_CLK, 0x0);

   /* SPI_1_MISO */
   pinmux_set_swmux(SWMUX_SDL_EIM_D17, MUX_CTL_MUX_MODE_ALT1);
   pinmux_set_padcfg(SWPAD_SDL_EIM_D17, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI1_IPP_IND_MISO, 0x0);
    
   /* SPI_1_MOSI */
   pinmux_set_swmux(SWMUX_SDL_EIM_D18, MUX_CTL_MUX_MODE_ALT1);
   pinmux_set_padcfg(SWPAD_SDL_EIM_D18, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI1_IPP_IND_MOSI, 0x0);
    
   /* SPI_1_CS# */
   pinmux_set_swmux(SWMUX_SDL_EIM_D19, MUX_CTL_MUX_MODE_ALT1);
   pinmux_set_padcfg(SWPAD_SDL_EIM_D19, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI1_IPP_IND_SS_B_1, 0x0);
    
#ifdef BOARD_TS4900
   /* SPI_2_CLK */
   pinmux_set_swmux(SWMUX_SDL_CSI0_DAT8, MUX_CTL_MUX_MODE_ALT0);
   pinmux_set_padcfg(SWPAD_SDL_CSI0_DAT8, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI2_IPP_CSPI_CLK, 0x0);

   /* SPI_2_MOSI */
   pinmux_set_swmux(SWMUX_SDL_CSI0_DAT9, MUX_CTL_MUX_MODE_ALT0);
   pinmux_set_padcfg(SWPAD_SDL_CSI0_DAT9, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI2_IPP_IND_MISO, 0x0);

   /* SPI_2_MISO */
   pinmux_set_swmux(SWMUX_SDL_CSI0_DAT10, MUX_CTL_MUX_MODE_ALT0);
   pinmux_set_padcfg(SWPAD_SDL_CSI0_DAT10, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI2_IPP_IND_MOSI, 0x0);
   /* SPI_2_CS# */
   pinmux_set_swmux(SWMUX_SDL_CSI0_DAT11, MUX_CTL_MUX_MODE_ALT0);
   pinmux_set_padcfg(SWPAD_SDL_CSI0_DAT11, MX6X_PAD_SETTINGS_ECSPI);
   pinmux_set_input(SWINPUT_SDL_ECSPI2_IPP_IND_SS_B_1, 0x0);
#endif    

}

void mx6sdl_init_can(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    /*
     * The code below initializes the i.MX6SDL CAN1 module pins,
     * and the corresponding MC33902 CAN transceiver chips.
     */

    /* CAN1 TX: signal name: KEY_COL2, pad name: KEY_COL2, connected to module can1's TXCAN pin */
    pinmux_set_swmux(SWMUX_SDL_KEY_COL2, MUX_CTL_MUX_MODE_ALT2);    

    /* CAN1 RX: signal name: KEY_ROW2, pad name: KEY_ROW2, connected to module can1's RXCAN pin */
    pinmux_set_swmux(SWMUX_SDL_KEY_ROW2, MUX_CTL_MUX_MODE_ALT2);
    pinmux_set_input(SWINPUT_SDL_CAN1_IPP_IND_CANRX, 1);
        
#ifdef BOARD_TS7970    
    // EN_CAN_1# on DISP0_DAT0, ALT5 = GPIO4[21]
    pinmux_set_swmux(SWMUX_SDL_DISP0_DAT0, MUX_CTL_MUX_MODE_ALT5);
    pinmux_set_padcfg(SWPAD_SDL_DISP0_DAT0, MX6X_PAD_SETTINGS_GPIO);
    out32(MX6X_GPIO4_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO4_BASE + MX6X_GPIO_GDIR) | (0x1 << 21));
    out32(MX6X_GPIO4_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO4_BASE + MX6X_GPIO_DR) & ~(0x1 << 21));
#endif    
    
    /* CAN2 TX: signal name: KEY_COL4, pad name: KEY_COL4, connected to module can2's TXCAN pin */
    pinmux_set_swmux(SWMUX_SDL_KEY_COL4, MUX_CTL_MUX_MODE_ALT0);    

    /* CAN2 RX: signal name: KEY_ROW4, pad name: KEY_ROW4, connected to module can2's RXCAN pin */
    pinmux_set_swmux(SWMUX_SDL_KEY_ROW4, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_input(SWINPUT_SDL_CAN2_IPP_IND_CANRX, 0);
    
#ifdef BOARD_TS7970    
    // EN_CAN_2# on EIM_BCLK, ALT5 = GPIO6[31]
    pinmux_set_swmux(SWMUX_SDL_EIM_BCLK, MUX_CTL_MUX_MODE_ALT5);
    pinmux_set_padcfg(SWPAD_SDL_EIM_BCLK, MX6X_PAD_SETTINGS_GPIO);      
    out32(MX6X_GPIO6_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO6_BASE + MX6X_GPIO_GDIR) | (0x1 << 31));
    out32(MX6X_GPIO6_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO6_BASE + MX6X_GPIO_DR) & ~(0x1 << 31));
#endif    
    /*
     * CAN1 NERR: signal name: CAN1_NERR, pad name: GPIO_7, connected to GPIO1[7] pin
     * active low error signal which is driven by the CAN transceiver chip (i.e. an _input_ to the i.MX6 Q)
     */
    //pinmux_set_swmux(SWMUX_SDL_GPIO_7, MUX_CTL_MUX_MODE_ALT5);
    //out32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR) & ~(1<<7));
    
    /*
     * CAN1 EN: signal name: CAN1_EN, pad name: GPIO_4, connected to GPIO1[4] pin
     * Code below also sets the EN signal to be high
     */
    //pinmux_set_swmux(SWMUX_SDL_GPIO_4, MUX_CTL_MUX_MODE_ALT5);
    //out32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR) | (1<<4));
    //out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) | (1<<4));
    
    /*
     * CAN1 STBY: signal CAN1_STBY, pad name: GPIO_2, connected to GPIO1[2] pin
     * The CAN transceiver chip has an active low standby pin.  Code below ensures that STBY is high
     * to prevent the transceiver chip from being in standby mode.
     */

    //pinmux_set_swmux(SWMUX_SDL_GPIO_2, MUX_CTL_MUX_MODE_ALT5);
    //out32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR) | (1<<2));
    //out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) | (1<<2));
}

void mx6sdl_init_lvds(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
#if (0)
    /* Enable PWM - Used to configure backlight brightness (GPIO1[18]) */
    pinmux_set_swmux(SWMUX_SDL_SD1_CMD, MUX_CTL_MUX_MODE_ALT5);
    
    /* 
     * Configure GPIO controlling PWM as an output and drive the GPIO high.  In this case PWM is always high, meaning
     * a 100% duty cycle, a lower duty cycle could be used to decrease the brightness of the display. 
     */
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR) | (0x1 << 18));
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) | (0x1 << 18));

    /* Set pad GPIO_9 mux mode to GPIO1[9] which allows an external LVDS display to send touch interrupts to the i.MX6Q */
    pinmux_set_swmux(SWMUX_SDL_GPIO_9, MUX_CTL_MUX_MODE_ALT5);
#endif

}
void mx6sdl_init_lcd_panel(void)
{
#ifdef BOARD_TS4900

#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif	

	/* IPU1 Display Interface 0 clock */
	pinmux_set_swmux(SWMUX_SDL_DI0_DISP_CLK, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_padcfg(SWPAD_SDL_DI0_DISP_CLK, MX6X_PAD_SETTINGS_IPU);

	/* LCD EN */
	pinmux_set_swmux(SWMUX_SDL_DI0_PIN15, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_padcfg(SWPAD_SDL_DI0_PIN15, MX6X_PAD_SETTINGS_IPU);

	/* LCD HSYNC */
	pinmux_set_swmux(SWMUX_SDL_DI0_PIN2, MUX_CTL_MUX_MODE_ALT0);
   pinmux_set_padcfg(SWPAD_SDL_DI0_PIN2, MX6X_PAD_SETTINGS_IPU);

	/* LCD VSYNC */
	pinmux_set_swmux(SWMUX_SDL_DI0_PIN3, MUX_CTL_MUX_MODE_ALT0);
   pinmux_set_padcfg(SWPAD_SDL_DI0_PIN3, MX6X_PAD_SETTINGS_IPU);

	/* Data Lines */
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT0, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT1, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT2, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT3, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT4, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT5, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT6, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT7, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT8, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT9, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT10, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT11, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT12, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT13, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT14, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT15, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT16, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT17, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT18, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT19, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT20, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT21, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT22, MUX_CTL_MUX_MODE_ALT0);
	pinmux_set_swmux(SWMUX_SDL_DISP0_DAT23, MUX_CTL_MUX_MODE_ALT0);
	

	/* EN_LCD_3.3V on EIM_A19, ALT5 = GPIO2[19]  */
	pinmux_set_swmux(SWMUX_SDL_EIM_A19, MUX_CTL_MUX_MODE_ALT5);
   pinmux_set_padcfg(SWPAD_SDL_EIM_A19, MX6X_PAD_SETTINGS_GPIO);
   /* EN_LCD_3.3V set high  */
	out32(MX6X_GPIO2_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO2_BASE + MX6X_GPIO_GDIR) | (0x1 << 19));
	out32(MX6X_GPIO2_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO2_BASE + MX6X_GPIO_DR) | (0x1 << 19));

   /* Enable PWM3 pin in GPIO mode - Used to configure backlight brightness (GPIO2[9]) - set to 100% duty cycle - full brightness */
	pinmux_set_swmux(SWMUX_SDL_SD4_DAT1, MUX_CTL_MUX_MODE_ALT5);
	out32(MX6X_GPIO2_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO2_BASE + MX6X_GPIO_GDIR) | (0x1 << 9));
	out32(MX6X_GPIO2_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO2_BASE + MX6X_GPIO_DR) | (0x1 << 9));

	
#endif    
}

void mx6sdl_init_misc_pins(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
    pinmux_set_swmux(SWMUX_SDL_EIM_A21, MUX_CTL_MUX_MODE_ALT5);   // DIO_12, GPIO2_17    same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_BCLK, MUX_CTL_MUX_MODE_ALT5);   // DIO_13, GPIO6_31   EN_CAN_2# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A18, MUX_CTL_MUX_MODE_ALT5);   // DIO_14, GPIO2_20    same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_OE, MUX_CTL_MUX_MODE_ALT5);   // DIO_15, GPIO2_25     MODBUS_FAULT on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A24, MUX_CTL_MUX_MODE_ALT5);   // DIO_20, GPIO5_4     same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA0, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_00, GPIO3_0  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA1, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_01, GPIO3_1  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA2, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_02, GPIO3_2  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA3, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_03, GPIO3_3  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA4, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_04, GPIO3_4  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA5, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_05, GPIO3_5  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA6, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_06, GPIO3_6  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA7, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_07, GPIO3_7  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA8, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_08, GPIO3_8  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA9, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_09, GPIO3_9  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA10, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_10, GPIO3_10  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA11, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_11, GPIO3_11  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA12, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_12, GPIO3_12  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA13, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_13, GPIO3_13  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA14, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_14, GPIO3_14  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_DA15, MUX_CTL_MUX_MODE_ALT5);   // MUX_AD_15, GPIO3_15  same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_LBA, MUX_CTL_MUX_MODE_ALT5);   // BUS_ALE#, GPIO2_27    same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_RW, MUX_CTL_MUX_MODE_ALT5);   // BUS_DIR, GPIO2_26      same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_CS0, MUX_CTL_MUX_MODE_ALT5);   // BUS_CS#, GPIO2_23     same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_WAIT, MUX_CTL_MUX_MODE_ALT5);   // BUS_WAIT#, GPIO5_0   same on TS-7970        
    pinmux_set_swmux(SWMUX_SDL_GPIO_0, MUX_CTL_MUX_MODE_ALT0);   // GPIO_0, CCM_CLK01, AUD_MCLK same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_2, MUX_CTL_MUX_MODE_ALT5);       // RED_LED#, GPIO1_2       same on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A16, MUX_CTL_MUX_MODE_ALT5);   // EN_USB_5V#, GPIO2_22       same on TS-7970
    
    pinmux_set_swmux(SWMUX_SDL_EIM_D23, MUX_CTL_MUX_MODE_ALT5);   // EN_RTC#, GPIO3_23       same on TS-7970
    out32(MX6X_GPIO3_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO3_BASE + MX6X_GPIO_GDIR) | (0x1 << 23));
    out32(MX6X_GPIO3_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO3_BASE + MX6X_GPIO_DR) & ~(0x1 << 23));  // Drive EN_RTC# low

#ifdef BOARD_TS4900    
    pinmux_set_swmux(SWMUX_SDL_CSI0_VSYNC, MUX_CTL_MUX_MODE_ALT5);   // FPGA_RESET#, GPIO5_21 n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_CSI0_DATA_EN, MUX_CTL_MUX_MODE_ALT5);   // FPGA_DONE, GPIO5_20 FPGA_IRQ_0 on TS-7970
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT12, MUX_CTL_MUX_MODE_ALT5);   // DIO_0, GPIO5_30  n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_5, MUX_CTL_MUX_MODE_ALT5);       // DIO_1, GPIO1_5   n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_6, MUX_CTL_MUX_MODE_ALT5);       // DIO_2, GPIO1_6   n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_16, MUX_CTL_MUX_MODE_ALT5);      // DIO_4, GPIO7_11  n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_19, MUX_CTL_MUX_MODE_ALT5);      // DIO_7, GPIO4_5   n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_CSI0_MCLK, MUX_CTL_MUX_MODE_ALT5);    // DIO_8, GPIO5_19  n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A20, MUX_CTL_MUX_MODE_ALT5);   // DIO_16, GPIO2_18    n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A22, MUX_CTL_MUX_MODE_ALT5);   // DIO_18, GPIO2_16    n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A23, MUX_CTL_MUX_MODE_ALT5);   // DIO_19, GPIO6_6     n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_CS1, MUX_CTL_MUX_MODE_ALT5);       // GREEN_LED#, GPIO2_24   n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_9, MUX_CTL_MUX_MODE_ALT5);       // DIO_3, GPIO1_9   YEL_LED# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_17, MUX_CTL_MUX_MODE_ALT5);      // DIO_5, GPIO7_12  FPGA_SPI_CS1# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT13, MUX_CTL_MUX_MODE_ALT5);   // DIO_6, GPIO5_31  FPGA_SPI_CS0# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_CSI0_PIXCLK, MUX_CTL_MUX_MODE_ALT5);  // DIO_9, GPIO5_18  HD1_SPI_CS# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_EB1, MUX_CTL_MUX_MODE_ALT5);      // DIO_10, GPIO2_29 BUS_BHE# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_D31, MUX_CTL_MUX_MODE_ALT5);   // BUS_BHE#, GPIO3_31    n/c on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A17, MUX_CTL_MUX_MODE_ALT5);   // OFF_BD_RESET#, GPIO2_21    I210_RESET# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_SD3_RST, MUX_CTL_MUX_MODE_ALT5);   // SD3_RST, GPIO7_8, CPU_DIO_B nc/ on TS-7970
#else
    pinmux_set_swmux(SWMUX_SDL_GPIO_9, MUX_CTL_MUX_MODE_ALT5);       // DIO_3, GPIO1_9   YEL_LED# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_GPIO_17, MUX_CTL_MUX_MODE_ALT5);      // DIO_5, GPIO7_12  FPGA_SPI_CS1# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT13, MUX_CTL_MUX_MODE_ALT5);   // DIO_6, GPIO5_31  FPGA_SPI_CS0# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_CSI0_PIXCLK, MUX_CTL_MUX_MODE_ALT5);  // DIO_9, GPIO5_18  HD1_SPI_CS# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_EB1, MUX_CTL_MUX_MODE_ALT5);      // DIO_10, GPIO2_29 BUS_BHE# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_EIM_A17, MUX_CTL_MUX_MODE_ALT5);   // OFF_BD_RESET#, GPIO2_21    I210_RESET# on TS-7970
    pinmux_set_swmux(SWMUX_SDL_DISP0_DAT4, MUX_CTL_MUX_MODE_ALT5);  // DISP0_DAT4, GPIO4_25, IMX6_BLUE_LED on TS-7970
#endif

    pinmux_set_swmux(SWMUX_SDL_GPIO_3, MUX_CTL_MUX_MODE_ALT3);   // FPGA_24MHZ, XTALOSC_REF_CLK_24M same on TS-7970
    pinmux_set_padcfg(SWPAD_SDL_GPIO_3, MX6SDL_PAD_SETTINGS_24MHZ_GPIO3);       
    pinmux_set_swmux(SWMUX_SDL_GPIO_4, MUX_CTL_MUX_MODE_ALT0);   // GPIO_4, GPIO1_4, FPGA_IRQ, FPGA_IRQ_1 on TS-7970
    pinmux_set_swmux(SWMUX_SDL_SD3_DAT4, MUX_CTL_MUX_MODE_ALT5);   // SD3_DAT4, unused pin n/c on both boards
    pinmux_set_swmux(SWMUX_SDL_SD3_DAT5, MUX_CTL_MUX_MODE_ALT5);   // SD3_DAT5, unused pin n/c on both boards    
}

#if (0)
 not used on TS boards...?
void mx6sdl_init_camera(void)
{
    
    // GPIO pin muxing
    pinmux_set_swmux(SWMUX_SDL_GPIO_3, MUX_CTL_MUX_MODE_ALT4);  
    pinmux_set_swmux(SWMUX_SDL_GPIO_6, MUX_CTL_MUX_MODE_ALT5);
    pinmux_set_swmux(SWMUX_SDL_GPIO_8, MUX_CTL_MUX_MODE_ALT5);
    pinmux_set_swmux(SWMUX_SDL_SD1_DAT0, MUX_CTL_MUX_MODE_ALT5);

    // CSI0 pin muxing
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT8, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT9, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT10, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT11, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT12, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT13, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT14, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT15, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT16, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT17, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT18, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_DAT19, MUX_CTL_MUX_MODE_ALT0);

    pinmux_set_swmux(SWMUX_SDL_CSI0_DATA_EN, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_MCLK, MUX_CTL_MUX_MODE_ALT0);   
    pinmux_set_swmux(SWMUX_SDL_CSI0_PIXCLK, MUX_CTL_MUX_MODE_ALT0);
    pinmux_set_swmux(SWMUX_SDL_CSI0_VSYNC, MUX_CTL_MUX_MODE_ALT0);  

    // configure CLKO2 to drive a 24MHz clock to the camera module
    out32(MX6X_CCM_BASE + MX6X_CCM_CCOSR, 0x010E00FB);

    // power down camera
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR) | (0x1 << 6));
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) | (0x1 << 6));
    mx6x_usleep(1000);
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) & ~(0x1 << 6));

    // reset camera
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_GDIR) | (0x1 << 8));
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) | (0x1 << 8));
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) & ~(0x1 << 8));
    mx6x_usleep(1000);
    out32(MX6X_GPIO1_BASE + MX6X_GPIO_DR, in32(MX6X_GPIO1_BASE + MX6X_GPIO_DR) | (0x1 << 8));

    // GPMC configuration
    out32(MX6X_IOMUXC_BASE + MX6X_IOMUX_GPR1, in32(MX6X_IOMUXC_BASE + MX6X_IOMUX_GPR1) | (1 << 19));
}
#endif

void mx6sdl_init_displays(void)
{
    mx6sdl_init_lcd_panel();    
    mx6sdl_init_lvds();
}


void mx6sdl_init_uart1(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
   pinmux_set_swmux(SWMUX_SDL_SD3_DAT6, MUX_CTL_MUX_MODE_ALT1);
   pinmux_set_padcfg(SWPAD_SDL_SD3_DAT6, MX6X_PAD_SETTINGS_UART);

   pinmux_set_swmux(SWMUX_SDL_SD3_DAT7, MUX_CTL_MUX_MODE_ALT1);
   pinmux_set_padcfg(SWPAD_SDL_SD3_DAT7, MX6X_PAD_SETTINGS_UART);
}

void mx6sdl_init_uart2(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
   pinmux_set_swmux(SWMUX_SDL_GPIO_7, MUX_CTL_MUX_MODE_ALT4);
   pinmux_set_padcfg(SWPAD_SDL_GPIO_7, MX6X_PAD_SETTINGS_UART);
   
   pinmux_set_swmux(SWMUX_SDL_GPIO_8, MUX_CTL_MUX_MODE_ALT4);
   pinmux_set_padcfg(SWPAD_SDL_GPIO_8, MX6X_PAD_SETTINGS_UART);
   
   pinmux_set_swmux(SWMUX_SDL_SD4_DAT6, MUX_CTL_MUX_MODE_ALT2);
   pinmux_set_padcfg(SWPAD_SDL_SD4_DAT6, MX6X_PAD_SETTINGS_UART);
   
   pinmux_set_swmux(SWMUX_SDL_SD4_DAT5, MUX_CTL_MUX_MODE_ALT2);
   pinmux_set_padcfg(SWPAD_SDL_SD4_DAT5, MX6X_PAD_SETTINGS_UART);      
}

void mx6sdl_init_uart3(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
   pinmux_set_swmux(SWMUX_SDL_EIM_D24, MUX_CTL_MUX_MODE_ALT2);
   pinmux_set_padcfg(SWPAD_SDL_EIM_D24, MX6X_PAD_SETTINGS_UART);
   
   pinmux_set_swmux(SWMUX_SDL_EIM_D25, MUX_CTL_MUX_MODE_ALT2);
   pinmux_set_padcfg(SWPAD_SDL_EIM_D25, MX6X_PAD_SETTINGS_UART);
}

void mx6sdl_init_uart4(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
   pinmux_set_swmux(SWMUX_SDL_KEY_COL0, MUX_CTL_MUX_MODE_ALT4);
   pinmux_set_padcfg(SWMUX_SDL_KEY_COL0, MX6X_PAD_SETTINGS_UART);
   
   pinmux_set_swmux(SWMUX_SDL_KEY_ROW0, MUX_CTL_MUX_MODE_ALT4);
   pinmux_set_padcfg(SWMUX_SDL_KEY_ROW0, MX6X_PAD_SETTINGS_UART);
}
void mx6sdl_init_uart5(void)
{
#if DEBUG_ON
   kprintf("%s\n", __func__);
#endif
   pinmux_set_swmux(SWMUX_SDL_KEY_COL1, MUX_CTL_MUX_MODE_ALT4);
   pinmux_set_padcfg(SWMUX_SDL_KEY_COL1, MX6X_PAD_SETTINGS_UART);
 
   pinmux_set_swmux(SWMUX_SDL_KEY_ROW1, MUX_CTL_MUX_MODE_ALT4);
   pinmux_set_padcfg(SWMUX_SDL_KEY_ROW1, MX6X_PAD_SETTINGS_UART);
}



#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn/product/branches/6.6.0/trunk/hardware/startup/boards/imx6x/sabrelite/mx6q_gpio.c $ $Rev: 712131 $")
#endif